from phonenumber_field.serializerfields import PhoneNumberField
from rest_framework import serializers

from garpix_user.mixins.serializers import ToLowerMixin


class RestoreByEmailSerializer(ToLowerMixin, serializers.Serializer):
    email = serializers.EmailField(required=True)


class RestoreByPhoneSerializer(serializers.Serializer):
    phone = PhoneNumberField(required=True)


class RestoreSetPasswordSerializer(serializers.Serializer):
    new_password = serializers.CharField(max_length=255, required=True)


class RestoreCheckCodeSerializer(serializers.Serializer):
    restore_confirmation_code = serializers.CharField(max_length=15, required=True)



from django.contrib.auth import get_user_model
from rest_framework.generics import CreateAPIView


from garpix_user.serializers import RegistrationSerializer

User = get_user_model()


class RegistrationView(CreateAPIView):
    queryset = User.objects.all()
    serializer_class = RegistrationSerializer


registration_view = RegistrationView.as_view()



from drf_spectacular.utils import extend_schema
from rest_framework import viewsets, status
from rest_framework.decorators import action
from rest_framework.exceptions import ValidationError
from rest_framework.response import Response
from django.utils.translation import ugettext as _

from garpix_user.exceptions import ValidationErrorSerializer
from garpix_user.models import UserSession
from garpix_user.serializers import RestoreByPhoneSerializer, UserSessionTokenSerializer
from garpix_user.serializers.restore_passwrod_serializer import RestoreCheckCodeSerializer, RestoreSetPasswordSerializer


class RestorePhonePasswordView(viewsets.ViewSet):

    def get_serializer_class(self):
        if self.action == 'send_code':
            return RestoreByPhoneSerializer
        if self.action == 'check_code':
            return RestoreCheckCodeSerializer
        return RestoreSetPasswordSerializer

    @extend_schema(
        summary='Restore password by phone. Step 1',
        responses={
            status.HTTP_200_OK: UserSessionTokenSerializer,
            status.HTTP_400_BAD_REQUEST: ValidationErrorSerializer
        }
    )
    @action(methods=['POST'], detail=False)
    def send_code(self, request, *args, **kwargs):
        user = UserSession.get_or_create_user_session(request)
        serializer = RestoreByPhoneSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        result = user.send_restore_code(phone=serializer.data['phone'])

        if result is not True:
            result.raise_exception(exception_class=ValidationError)
        return Response(UserSessionTokenSerializer(user).data)

    @extend_schema(
        summary='Restore password by phone. Step 2',
        responses={
            status.HTTP_204_NO_CONTENT: '',
            status.HTTP_400_BAD_REQUEST: ValidationErrorSerializer
        }
    )
    @action(methods=['POST'], detail=False)
    def check_code(self, request, *args, **kwargs):
        user = UserSession.get_or_create_user_session(request)

        serializer = RestoreCheckCodeSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        result = user.check_restore_code(restore_confirmation_code=serializer.data['restore_confirmation_code'])

        if result is not True:
            result.raise_exception(exception_class=ValidationError)
        return Response(status=status.HTTP_204_NO_CONTENT)

    @extend_schema(
        summary='Restore password by phone. Step 3',
        responses={
            status.HTTP_200_OK: _('Password was updated!'),
            status.HTTP_400_BAD_REQUEST: ValidationErrorSerializer
        }
    )
    @action(methods=['POST'], detail=False)
    def set_password(self, request, *args, **kwargs):
        user = UserSession.get_or_create_user_session(request)

        serializer = RestoreSetPasswordSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        result = user.restore_password(new_password=serializer.data['new_password'])

        if result is not True:
            result.raise_exception(exception_class=ValidationError)
        return Response(_('Password was updated!'))



from django.conf import settings
from django.contrib.auth import get_user_model
from rest_framework import serializers


User = get_user_model()


class RegistrationSerializer(serializers.ModelSerializer):
    password = serializers.CharField(write_only=True)
    password_2 = serializers.CharField(write_only=True)

    def validate(self, data):
        min_length = getattr(settings, 'MIN_LENGTH_PASSWORD', 8)
        min_digits = getattr(settings, 'MIN_DIGITS_PASSWORD', 2)
        min_chars = getattr(settings, 'MIN_CHARS_PASSWORD', 2)
        min_uppercase = getattr(settings, 'MIN_UPPERCASE_PASSWORD', 1)

        queryset = User.objects.filter(email=data['email']).first()
        if queryset is not None:
            raise serializers.ValidationError("This email is already in use")

        # check for 2 min length
        if len(data['password']) < min_length:
            raise serializers.ValidationError(
                'Password must be at least {min_length} characters long.'.format(min_length=min_length)
            )

        # check for 2 digits
        if sum(c.isdigit() for c in data['password']) < min_digits:
            raise serializers.ValidationError(
                'Password must container at least {min_digits} digits.'.format(min_digits=min_digits)
            )

        # check for 2 char
        if sum(c.isalpha() for c in data['password']) < min_chars:
            raise serializers.ValidationError(
                'Password must container at least {min_chars} chars.'.format(min_chars=min_chars)
            )

        # check for uppercase letter
        if sum(c.isupper() for c in data['password']) < min_uppercase:
            raise serializers.ValidationError(
                'Password must container at least {min_uppercase} uppercase letter.'.format(min_uppercase=min_uppercase)
            )

        # check for password = password_2
        if data['password'] != data['password_2']:
            raise serializers.ValidationError("Passwords do not match")

        return data

    def create(self, validated_data):
        user = User.objects.create_user(
            username=validated_data['username'],
            password=validated_data['password'],
            email=validated_data['email'],
        )

        return user

    class Meta:
        model = User
        fields = ('username', 'email', 'password', 'password_2',)



import uuid
from django.db import models
from django.conf import settings
from django.utils import timezone
from django.contrib.auth import get_user_model
from django.utils.module_loading import import_string

UserSessionMixin = import_string(
    settings.GARPIX_USER.get('USER_USERSESSION_MIXIN', 'garpix_user.mixins.models.user_session.UserSessionMixin'))


class UserSession(UserSessionMixin, models.Model):
    HEAD_NAME = 'UserSession-Token'

    class UserState(models.IntegerChoices):
        UNRECOGNIZED = (0, 'Неопознанный')
        GUEST = (1, 'Гость')
        REGISTERED = (2, 'Зарегистрированный')

    user = models.OneToOneField(
        settings.AUTH_USER_MODEL,
        null=True,
        blank=True,
        on_delete=models.CASCADE,
        verbose_name='Пользователь'
    )
    token_number = models.CharField(max_length=256, null=True, blank=True, verbose_name='user token')
    recognized = models.PositiveIntegerField(
        default=UserState.UNRECOGNIZED,
        choices=UserState.choices,
        verbose_name='Тип',
        help_text='Обозначает состояние, в котором распознается пользователь.'
    )
    last_access = models.DateTimeField(
        'Последний вход',
        default=timezone.now,
    )

    @classmethod
    def get_from_request(cls, request):
        user = request.user
        if user.is_authenticated:
            return UserSession.objects.filter(user=user).first()

        token = request.headers.get(cls.HEAD_NAME, None)
        if token is not None:
            return UserSession.objects.filter(token_number=token).first()

        token = request.session.session_key
        if token is not None:
            return UserSession.objects.filter(token_number=token).first()
        email = request.GET.get('email', None)
        if email is not None:
            return UserSession.objects.filter(user__email=email).first()
        phone = request.GET.get('phone', None)
        if phone is not None:
            return UserSession.objects.filter(user__phone=phone).first()
        return None

    @classmethod
    def set_user_from_request(cls, request):
        user_session = cls.get_from_request(request)
        if request.user.is_authenticated and user_session is not None:
            user = get_user_model().objects.get(pk=request.user.pk)
            user_session.user = user
            user_session.save()
            return True
        return False

    @classmethod
    def get_or_create_user_session(cls, request, session=False):
        User = get_user_model()

        user_session = cls.get_from_request(request)
        if user_session is not None:
            return user_session

        if request.user.is_authenticated:
            user = get_user_model().objects.get(pk=request.user.pk)
            return UserSession.objects.create(
                user=user,
                recognized=UserSession.UserState.REGISTERED
            )

        if session is True:
            token = request.session.session_key
            return UserSession.objects.create(
                token_number=token,
                recognized=UserSession.UserState.GUEST
            )

        email = request.GET.get('email', None)
        if email is not None:
            try:
                user = User.objects.get(email=email)
                return UserSession.objects.create(
                    token_number=uuid.uuid4(),
                    recognized=UserSession.UserState.REGISTERED,
                    user=user
                )
            except Exception:
                pass

        phone = request.GET.get('phone', None)
        if phone is not None:
            try:
                user = User.objects.get(phone=phone)
                return UserSession.objects.create(
                    token_number=uuid.uuid4(),
                    recognized=UserSession.UserState.REGISTERED,
                    user=user
                )
            except Exception:
                pass

        return UserSession.objects.create(
            token_number=uuid.uuid4(),
            recognized=UserSession.UserState.GUEST
        )

    class Meta:
        verbose_name = 'Пользователь системы'
        verbose_name_plural = 'Пользователи системы'

    def __str__(self):
        return f'{self.pk}'



AUTHENTICATION_BACKENDS = (
    'social_core.backends.google.GoogleOAuth2',
    'social_core.backends.twitter.TwitterOAuth',
    'social_core.backends.vk.VKOAuth2',
    'social_core.backends.facebook.FacebookAppOAuth2',
    'social_core.backends.facebook.FacebookOAuth2',
    # Django
    'rest_framework_social_oauth2.backends.DjangoOAuth2',
    'django.contrib.auth.backends.ModelBackend',
)
SOCIAL_AUTH_PIPELINE = (
    'social_core.pipeline.social_auth.social_details',
    'social_core.pipeline.social_auth.social_uid',
    'social_core.pipeline.social_auth.auth_allowed',
    'social_core.pipeline.social_auth.social_user',
    'social_core.pipeline.user.get_username',
    'social_core.pipeline.social_auth.associate_by_email',
    'social_core.pipeline.user.create_user',
    'social_core.pipeline.social_auth.associate_user',
    'social_core.pipeline.social_auth.load_extra_data',
    'social_core.pipeline.user.user_details'
)

PHONE_CONFIRMATION_EVENT = 2021
EMAIL_CONFIRMATION_EVENT = 2022

PHONE_RESTORE_PASSWORD_EVENT = 2023
EMAIL_RESTORE_PASSWORD_EVENT = 2024
EMAIL_LINK_CONFIRMATION_EVENT = 2025

PHONE_CONFIRMATION_EVENT_ITEM = {
    PHONE_CONFIRMATION_EVENT: {
        'title': 'Подтверждение номера телефона',
        'context_description': '{{ confirmation_code }}'
    }
}

EMAIL_CONFIRMATION_EVENT_ITEM = {
    EMAIL_CONFIRMATION_EVENT: {
        'title': 'Подтверждение email',
        'context_description': '{{ confirmation_code }}'
    }
}

PHONE_RESTORE_PASSWORD_EVENT_ITEM = {
    PHONE_RESTORE_PASSWORD_EVENT: {
        'title': 'Восстановление пароля по смс',
        'context_description': '{{ confirmation_code }}'
    }
}

EMAIL_RESTORE_PASSWORD_EVENT_ITEM = {
    EMAIL_RESTORE_PASSWORD_EVENT: {
        'title': 'Восстановление пароля по email',
        'context_description': '{{ confirmation_code }}'
    }
}

EMAIL_LINK_CONFIRMATION_EVENT_ITEM = {
    EMAIL_LINK_CONFIRMATION_EVENT: {
        'title': 'Подтверждение email по ссылке',
        'link': '{{ link }}'
    }
}

GARPIX_CONFIRM_CODE_LENGTH = 6
GARPIX_TIME_LAST_REQUEST = 2
GARPIX_CONFIRM_PHONE_CODE_LIFE_TIME = 5  # in minutes
GARPIX_CONFIRM_EMAIL_CODE_LIFE_TIME = 2  # in days

NOTIFY_EVENTS = {}

NOTIFY_EVENTS.update(PHONE_CONFIRMATION_EVENT_ITEM)
NOTIFY_EVENTS.update(EMAIL_CONFIRMATION_EVENT_ITEM)

NOTIFY_EVENTS.update(PHONE_RESTORE_PASSWORD_EVENT_ITEM)
NOTIFY_EVENTS.update(EMAIL_RESTORE_PASSWORD_EVENT_ITEM)
NOTIFY_EVENTS.update(EMAIL_LINK_CONFIRMATION_EVENT_ITEM)

CHOICES_NOTIFY_EVENT = [(k, v['title']) for k, v in NOTIFY_EVENTS.items()]

# registration
GARPIX_USER_CONFIG = 'garpix_user.models.users_config.GarpixUserConfig'
GARPIX_USE_PREREGISTRATION_EMAIL_CONFIRMATION = True
GARPIX_USE_PREREGISTRATION_PHONE_CONFIRMATION = True

MIN_LENGTH_PASSWORD = 8
MIN_DIGITS_PASSWORD = 2
MIN_CHARS_PASSWORD = 2
MIN_UPPERCASE_PASSWORD = 1



from django.conf import settings
from django.db import models
from django.utils.translation import gettext as _
from garpix_utils.string import get_random_string

import string
from datetime import datetime, timezone, timedelta


class RestorePasswordMixin(models.Model):
    restore_password_confirm_code = models.CharField(_('Restore password code'),
                                                     max_length=15, null=True, blank=True)
    is_restore_code_confirmed = models.BooleanField(_('Restore code confirmed'), default=False)
    restore_date = models.DateTimeField(_('Restore code sent date'), null=True)

    def _check_user_data(self, email, phone):
        from garpix_user.exceptions import UserUnregisteredException

        if email and (self.user is None or email != self.user.email or not self.user.is_email_confirmed):
            UserUnregisteredException(field='email',
                                      extra_data={'field': self._meta.get_field('email').verbose_name.title().lower()})

        if phone and (self.user is None or phone != self.user.phone or not self.user.is_phone_confirmed):
            UserUnregisteredException(field='phone',
                                      extra_data={'field': self._meta.get_field('phone').verbose_name.title().lower()})

    def _check_request_time(self):
        from garpix_user.exceptions import WaitException

        if (TIME_LAST_REQUEST := settings.GARPIX_USER.get('TIME_LAST_REQUEST', None)) and (restore_date := self.restore_date):
            if restore_date + timedelta(minutes=TIME_LAST_REQUEST) >= datetime.now(restore_date.tzinfo):
                return WaitException()

    def send_restore_code(self, email=None, phone=None):
        from garpix_notify.models import Notify

        self._check_user_data(email, phone)
        self._check_request_time()

        confirmation_code = get_random_string(settings.GARPIX_USER.get('CONFIRM_CODE_LENGTH', 6), string.digits)

        self.restore_password_confirm_code = confirmation_code
        self.restore_date = datetime.now(timezone.utc)
        self.save()

        if email:
            Notify.send(settings.EMAIL_RESTORE_PASSWORD_EVENT, {
                'user_fullname': str(self.user),
                'email': self.user.email,
                'restore_code': self.restore_password_confirm_code
            }, email=self.user.email)
        elif phone:
            Notify.send(settings.PHONE_RESTORE_PASSWORD_EVENT, {
                'user_fullname': str(self.user),
                'phone': self.user.phone,
                'restore_code': self.user.restore_password_confirm_code
            }, phone=self.user.phone)

        return True

    def check_restore_code(self, restore_password_confirm_code=None):
        from garpix_user.exceptions import IncorrectCodeException, NoTimeLeftException

        if self.restore_password_confirm_code != restore_password_confirm_code:
            return IncorrectCodeException(field='restore_password_confirm_code')

        time_is_up = (datetime.now(
            self.restore_date.tzinfo) - self.restore_date).seconds / 60 > settings.GARPIX_USER.get('CONFIRM_EMAIL_CODE_LIFE_TIME', 6)

        if time_is_up:
            return NoTimeLeftException(field='restore_password_confirm_code')

        self.is_restore_code_confirmed = True
        self.save()

        return True

    def restore_password(self, new_password=None):
        if self.is_restore_code_confirmed:
            self.user.set_password(new_password)
            self.user.save()
            return True
        return False

    class Meta:
        abstract = True
